
function []=GridBuilder()

fname=['EULV_PV3c3']
data_file='data_file_EULVshortNoLooseEnds.xlsx'

modelExist=exist(fname);

% Check if the file already exists and delete it if it does
if modelExist == 4
    % If it does then check whether it's open
    if bdIsLoaded(fname)
        % If it is then close it (without saving!)
        close_system(fname,0)
    end
    % delete the file
    delete([fname,'.slx']);
end

new_system(fname)
open_system(fname)

%% Add ssubsystem for measurements
%add_block('lib/Data',fname);

%% Bus placement
% ......... Import data ..............

bus_position=xlsread(data_file,'Buscoords');

% ..... Block Size: Bus bar/ Three phase measurement (Bus).....

width_bus=5;
height_bus=40;

% ..... Bus positions ..........

Bus_No=bus_position(:,1);
bus_X_position=bus_position(:,2);
bus_Y_position=bus_position(:,3);
bus_W_position=bus_X_position+ repmat(width_bus,length(bus_X_position),1);
bus_H_position=bus_Y_position+repmat(height_bus,length(bus_Y_position),1);

bus_coordinate=[bus_X_position bus_Y_position...
    bus_W_position bus_H_position];

% ..... Generate buses on the simulink model .........

for i=1:length(Bus_No)    
    Bus_placement(fname, Bus_No(i),bus_coordinate(i,:));
end

%% ......... Line placement ........
% ......... import data ..............

line_node = xlsread(data_file,'Lines');

% ......... Line conections ..............

bus_A=line_node(:,1);
bus_B=line_node(:,2);
line_length=line_node(:,4);     

% ..... Block Size: Cable/line (L).....

width_line=100;
height_line=50;

for i=1:length(bus_A)
    
    % .... Calculate the position of the lines using bus coordinates .....
    if bus_A(i)<=9
        bus_A_name=[fname '/B00' num2str(bus_A(i))];
    elseif bus_A(i)<=99
        bus_A_name=[fname '/B0' num2str(bus_A(i))];
    else
        bus_A_name=[fname '/B' num2str(bus_A(i))];
    end
    bus_A_position=get_param(bus_A_name,'Position');
    
    if bus_B(i)<=9
        bus_B_name=[fname '/B00' num2str(bus_B(i))];
    elseif bus_B(i)<=99
        bus_B_name=[fname '/B0' num2str(bus_B(i))];
    else
        bus_B_name=[fname '/B' num2str(bus_B(i))];
    end
    bus_B_position=get_param(bus_B_name,'Position');
    
    line_X_position=(bus_A_position(1)+bus_B_position(1))/2-(width_line/2);
    line_Y_position=(bus_A_position(2)+bus_B_position(2))/2;
    line_W_position=line_X_position+width_line;
    line_H_position=line_Y_position+height_line;
    line_position(i,:)=[line_X_position line_Y_position...
        line_W_position line_H_position];    
    
    % ............  determine whether to flip the bus and the line or not 
    
    if bus_A_position(1)>= bus_B_position(1)
        
        flip_bus_B(i)=1;    % flip the bus
        flip_line(i)=1;     % flip the line
        
    else
        
        flip_bus_B(i)=0;    % do not flip the bus        
        flip_line(i)=0;     % do not flip the line
        
    end
end
    
    % ..determine whether to rotate the Line or not and the orientation..
    
    delta_X=abs(bus_A_position(1)-bus_B_position(1));    
    
    if (delta_X < width_line+70) && (bus_B_position(2)<bus_A_position(2)) 
        
        rotate_line(i)=1;      %   Counter clock wise rotation ('up')
        
    elseif (delta_X < width_line+70) &&...
            (bus_B_position(2)>bus_A_position(2))
        
        rotate_line(i)=2;      %   Clock wise rotation ('down')        
        
    else
        
        rotate_line(i)=0;      %   Do nothing
        
    end
      
    % .... generate line ....

for j=1:length(bus_A)
    if bus_A(j)<=9
        Bus_A_Number=['00' num2str(bus_A(j))];
    elseif bus_A(j)<=99
        Bus_A_Number=['0' num2str(bus_A(j))];
    else
        Bus_A_Number=[num2str(bus_A(j))];
    end
    if bus_B(j)<=9
        Bus_B_Number=['00' num2str(bus_B(j))];
    elseif bus_B(j)<=99
        Bus_B_Number=['0' num2str(bus_B(j))];
    else
        Bus_B_Number=[num2str(bus_B(j))];   
    end
    Line_placement(fname, Bus_A_Number,Bus_B_Number,line_position(j,:),...
        flip_bus_B(j),flip_line(j), rotate_line(j));
        
end

%%  ............ Connect buses and lines .............

for j=1:length(bus_A)
    if bus_A(j)<=9
        Bus_A_Number=['00' num2str(bus_A(j))];
    elseif bus_A(j)<=99
        Bus_A_Number=['0' num2str(bus_A(j))];
    else
        Bus_A_Number=[num2str(bus_A(j))];
    end
    if bus_B(j)<=9
        Bus_B_Number=['00' num2str(bus_B(j))];
    elseif bus_B(j)<=99
        Bus_B_Number=['0' num2str(bus_B(j))];
    else
        Bus_B_Number=[num2str(bus_B(j))];   
    end
    Connect_Bus_Line(fname, Bus_A_Number,Bus_B_Number);    
    
end

%% ............ Load Placement ...............
% ..... Block Size: Bus , Load .....

width_bus=5;
height_bus=60;

width_load=40;
height_load=40;

% ..... Import data ........

spot_load_data= xlsread(data_file,'Loads');

load_node=spot_load_data;

for i=1:length(load_node(:,1))
    if load_node(i,2)<=9
        bus_dest=[fname '/B00' num2str(load_node(i,2))];
    elseif load_node(i,2)<=99
        bus_dest=[fname '/B0' num2str(load_node(i,2))];
    else
        bus_dest=[fname '/B' num2str(load_node(i,2))];
    end
    
    bus_position_L(i,:)=get_param(bus_dest,'Position');
    Bus_orientation=get_param(bus_dest,'Orientation');
    
    if length(Bus_orientation)==4
    
        offset_X(i)=-60-width_load;   % when bus orientation : left               
        flip_load(i)=1;               % flip the Load
        
    else
    
        offset_X(i)=60;             % when bus orientation : right
        flip_load(i)=0;             % Do not flip the load
        
    end
    
    offset_Y(i)=0;%height_bus+20;
    
end

load_X_position=bus_position_L(:,1)+offset_X';
load_Y_position=bus_position_L(:,2)+offset_Y';
load_W_position=load_X_position+ repmat(width_load,...
    length(load_X_position), 1);
load_H_position=load_Y_position+repmat(height_load,...
    length(load_Y_position), 1);

load_coordinate=[load_X_position load_Y_position...
    load_W_position load_H_position];

% ..... Generate loads on the simulink model .........

for i=1:length(load_node(:,1))
    
    Load_Placement(fname, load_node(i,2),load_coordinate(i,:),...
        flip_load(i),load_node(i,1)); %
        
end

%%  ............ Connect buses and Loads .............

for i=1:length(load_node(:,1))
    
    Connect_Bus_Load(fname, load_node(i,2));    
    
end

ParamAss(fname);

end